-- Interfaces.C.Strings:
--
--   - Conversion de string C a Ada
--      C_String_Ptr: Interfaces.C.Strings.Chars_Ptr;
--      Ada_String  : String:=Interfaces.C.strings.Value(C_String_Ptr);
--                            ^^^^^^^^^^^^^^^^^^^^^^^^^^
--   - Null
--      Interfaces.C.Strings.Null_Ptr
--
--   - Length
--      Interfaces.C.Strings.StrLen
--
--   - Data
--      Interfaces.C.Strings.Chars_Ptr
--      Interfaces.C.Strings.New_Char_Array
--      Interfaces.C.Strings.New_String


with Interfaces.C.Strings;
private package Unix.System_Calls is 
   use Interfaces.C;
   use Interfaces.C.Strings;

   function Get_Errno return Int;
   procedure Perror (Arg: Chars_Ptr := Null_Ptr);

   -- Files system calls
   function Creat(Path : Chars_Ptr;
		  Perms: Int) return Int;

   function Open(Path : Chars_Ptr;
		 Flags: Int) return Int;

   -- Flags values:
   O_RDONLY: constant := 0;
   O_WRONLY: constant := 1;
   O_RDWR  : constant := 2;

   function Write(Fid   : Int;
		  Buff  : Chars_Ptr;
		  Nbytes: Int) return Int;

   function Read(Fid   : Int;
		 Buff  : Chars_ptr;
		 Nbytes: Int) return Int;

   function Close(Fid: Int) return Int;

   -- -----------------------------------------------------------------------

   function Lseek(Fid   : Int;
		  Offset: Long;
		  Whence: Int) return Int;

   -- Whence values
   SEEK_SET: constant:= 0;
   SEEK_CUR: constant:= 1;
   SEEK_END: constant:= 2;

   function Dup(Fid: Int) return Int;

   -- Processes system calls
   function Fork return Int;

   function Execlp(Path: Chars_Ptr := Null_Ptr;
		   Arg0: Chars_Ptr := Null_Ptr;
		   Arg1: Chars_Ptr := Null_Ptr;
		   Arg2: Chars_Ptr := Null_Ptr;
		   Arg3: Chars_Ptr := Null_Ptr;
		   Arg4: Chars_Ptr := Null_Ptr;
		   Arg5: Chars_Ptr := Null_Ptr;
		   Arg6: Chars_Ptr := Null_Ptr;
		   Arg7: Chars_Ptr := Null_Ptr;
		   Arg8: Chars_Ptr := Null_Ptr;
		   Arg9: Chars_Ptr := Null_Ptr) return Int;

--   type T_Argv is array(Positive range <>) of T_Str_Ptr;
--   function Execvp(File: String;
--		   Argv: T_Argv) return Int;

   function Wait(Status: Unix.Natural_Ptr) return Int;
   procedure Sys_Exit(Status: Int);

   -- Processes communication
   function Pipe(Fildes: Unix.T_Fildes) return Int;

private
   pragma Import (C, Get_Errno, "__get_errno", "__get_errno");
   pragma Import (C, Perror, "perror", "perror");
   pragma Import (C, Creat, "creat", "creat");
   pragma Import (C, Open, "open", "open");
   pragma Import (C, Read, "read", "read");
   pragma Import (C, Write, "write", "write");
   pragma Import (C, Close, "close", "close");
   pragma Import (C, Lseek, "lseek", "lseek");
   pragma Import (C, Dup, "dup", "dup");
   pragma Import (C, Fork, "fork", "fork");
   pragma Import (C, Execlp, "execlp", "execlp");
--   pragma Import (C, Execvp, "execvp", "execvp");
   pragma Import (C, Wait, "wait", "wait");
   pragma Import (C, Sys_Exit, "exit", "exit");
   pragma Import (C, Pipe, "pipe", "pipe");
end Unix.System_Calls;
